"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.strWordCount = exports.strTitle = exports.strSwap = exports.strStudly = exports.strStartsWith = exports.strStart = exports.strSquish = exports.strSnake = exports.strSlug = exports.strReplaceLast = exports.strReplaceFirst = exports.replaceArray = exports.replaceStr = exports.remove = exports.password = exports.mask = exports.lower = exports.limit = exports.lcFirst = exports.kebabCase = exports.isUuid = exports.isUlid = exports.isUrl = exports.isJson = exports.is = exports.headline = exports.finish = exports.endsWith = exports.containsAll = exports.contains = exports.camelCase = exports.etweenFirst = exports.between = exports.beforeLast = exports.before = exports.ascii = exports.afterLast = exports.after = exports.ucFirst = void 0;
var ucFirst = function (inputString) {
    if (!inputString || inputString.length === 0) {
        // Return an empty string if the input is null, undefined, or an empty string.
        return '';
    }
    var firstChar = inputString.charAt(0);
    // Check if the first character is already uppercase.
    if (firstChar === firstChar.toUpperCase()) {
        // If it is, no need to make any changes, return the original string.
        return inputString;
    }
    // Convert the first character to uppercase and concatenate it with the rest of the string.
    return firstChar.toUpperCase() + inputString.slice(1);
};
exports.ucFirst = ucFirst;
var after = function (haystack, needle) {
    var index = haystack.indexOf(needle);
    if (index === -1) {
        // If the needle is not found in the haystack, return an empty string.
        return '';
    }
    // Extract the substring after the needle and return it.
    return haystack.substring(index + needle.length);
};
exports.after = after;
var afterLast = function (haystack, needle) {
    var index = haystack.lastIndexOf(needle);
    if (index === -1 || index === haystack.length - 1) {
        // If the needle is not found in the haystack or it's the last character of the string,
        // return the entire string.
        return haystack;
    }
    // Extract the substring after the last occurrence of the needle and return it.
    return haystack.substring(index + needle.length);
};
exports.afterLast = afterLast;
var ascii = function (input) {
    return input
        .normalize("NFD") // Normalize Unicode string to decomposed form
        .replace(/[\u0300-\u036f]/g, ""); // Remove diacritical marks (accents)
};
exports.ascii = ascii;
var before = function (haystack, needle) {
    var index = haystack.indexOf(needle);
    if (index === -1) {
        // If the needle is not found in the haystack, return the entire string.
        return haystack;
    }
    // Extract the substring before the needle and return it.
    return haystack.substring(0, index);
};
exports.before = before;
var beforeLast = function (haystack, needle) {
    var index = haystack.lastIndexOf(needle);
    if (index === -1) {
        // If the needle is not found in the haystack, return the entire string.
        return haystack;
    }
    // Extract the substring before the last occurrence of the needle and return it.
    return haystack.substring(0, index);
};
exports.beforeLast = beforeLast;
var between = function (haystack, start, end) {
    var startIndex = haystack.indexOf(start);
    var endIndex = haystack.lastIndexOf(end);
    if (startIndex === -1 || endIndex === -1 || endIndex < startIndex) {
        // If either start or end is not found, or the end appears before the start, return an empty string.
        return '';
    }
    // Extract the substring between the start and end (excluding both start and end) and return it.
    return haystack.substring(startIndex + start.length, endIndex);
};
exports.between = between;
var etweenFirst = function (haystack, start, end) {
    var startIndex = haystack.indexOf(start);
    if (startIndex === -1) {
        // If the start is not found, return an empty string.
        return '';
    }
    var endIndex = haystack.indexOf(end, startIndex + start.length);
    if (endIndex === -1) {
        // If the end is not found, return an empty string.
        return '';
    }
    // Extract the substring between the start and end (excluding both start and end) and return it.
    return haystack.substring(startIndex + start.length, endIndex);
};
exports.etweenFirst = etweenFirst;
var camelCase = function (inputString) {
    return inputString.replace(/_([a-z])/g, function (_, match) { return match.toUpperCase(); });
};
exports.camelCase = camelCase;
var contains = function (haystack, needles) {
    if (typeof needles === 'string') {
        // Convert a single needle to an array for consistent handling.
        needles = [needles];
    }
    return needles.some(function (needle) { return haystack.includes(needle); });
};
exports.contains = contains;
var containsAll = function (haystack, needles) {
    if (typeof needles === 'string') {
        // Convert a single needle to an array for consistent handling.
        needles = [needles];
    }
    return needles.every(function (needle) { return haystack.includes(needle); });
};
exports.containsAll = containsAll;
var endsWith = function (haystack, needles) {
    if (typeof needles === 'string') {
        // Convert a single needle to an array for consistent handling.
        needles = [needles];
    }
    return needles.some(function (needle) { return haystack.endsWith(needle); });
};
exports.endsWith = endsWith;
var finish = function (inputString, ending) {
    return inputString.endsWith(ending) ? inputString : inputString + ending;
};
exports.finish = finish;
var headline = function (inputString) {
    return inputString
        .replace(/([a-z])([A-Z])/g, '$1 $2') // Insert space between lower and upper case letters
        .replace(/[_-]/g, ' ') // Replace underscores and hyphens with spaces
        .replace(/\b\w/g, function (char) { return char.toUpperCase(); }); // Capitalize the first letter of each word
};
exports.headline = headline;
var is = function (pattern, value) {
    var patternRegex = new RegExp('^' + pattern.split('*').join('.*') + '$');
    return patternRegex.test(value);
};
exports.is = is;
var isJson = function (input) {
    try {
        JSON.parse(input);
        return true;
    }
    catch (error) {
        return false;
    }
};
exports.isJson = isJson;
var isUrl = function (input) {
    var urlPattern = /^(https?|ftp):\/\/[^\s/$.?#].[^\s]*$/i;
    return urlPattern.test(input);
};
exports.isUrl = isUrl;
var isUlid = function (input) {
    var ulidPattern = /^[0-9A-Z]{26}$/i;
    return ulidPattern.test(input);
};
exports.isUlid = isUlid;
var isUuid = function (input) {
    var uuidPattern = /^[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}$/;
    return uuidPattern.test(input);
};
exports.isUuid = isUuid;
var kebabCase = function (input) {
    return input
        .replace(/([a-z])([A-Z])/g, '$1-$2') // Insert hyphen between lower and upper case letters
        .toLowerCase(); // Convert the entire string to lowercase
};
exports.kebabCase = kebabCase;
var lcFirst = function (input) {
    if (input.length === 0) {
        return input;
    }
    var firstChar = input.charAt(0).toLowerCase();
    return firstChar + input.slice(1);
};
exports.lcFirst = lcFirst;
var limit = function (input, maxLength, suffix) {
    if (suffix === void 0) { suffix = '...'; }
    if (input.length <= maxLength) {
        return input;
    }
    return input.substring(0, maxLength) + suffix;
};
exports.limit = limit;
var lower = function (input) { return input.toLocaleLowerCase(); };
exports.lower = lower;
var mask = function (input, maskChar, start, length) {
    if (length === void 0) { length = 0; }
    if (start >= 0) {
        var maskedPart = maskChar.repeat(length);
        return input.slice(0, start) + maskedPart + input.slice(start + length);
    }
    else {
        var maskedPart = maskChar.repeat(length);
        return input.slice(0, input.length + start) + maskedPart + input.slice(input.length);
    }
};
exports.mask = mask;
var password = function (length) {
    if (length === void 0) { length = 32; }
    var chars = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789~!@#$%^&*_-+=`|(){}[]:;<>,.?/';
    var pass = '';
    for (var i = 0; i < length; i++) {
        var randomIndex = Math.floor(Math.random() * chars.length);
        pass += chars[randomIndex];
    }
    return pass;
};
exports.password = password;
var remove = function (search, input, ignoreCase) {
    if (ignoreCase === void 0) { ignoreCase = false; }
    var flags = ignoreCase ? 'gi' : 'g';
    var regex = new RegExp(search instanceof Array ? search.join('|') : search, flags);
    return input.replace(regex, '');
};
exports.remove = remove;
var replaceStr = function (search, replace, input, caseSensitive) {
    if (caseSensitive === void 0) { caseSensitive = true; }
    var flags = caseSensitive ? 'g' : 'gi';
    var regex = search instanceof RegExp ? new RegExp(search.source, flags) : new RegExp(search, flags);
    return input.replace(regex, replace);
};
exports.replaceStr = replaceStr;
var replaceArray = function (search, replacements, input) {
    var index = 0;
    return input.replace(new RegExp(search, 'g'), function () { return replacements[index++]; });
};
exports.replaceArray = replaceArray;
var strReplaceFirst = function (search, replace, input) {
    var index = input.indexOf(search);
    if (index === -1) {
        return input; // Return the original string if the search value is not found.
    }
    return input.substring(0, index) + replace + input.substring(index + search.length);
};
exports.strReplaceFirst = strReplaceFirst;
var strReplaceLast = function (search, replace, input) {
    var index = input.lastIndexOf(search);
    if (index === -1) {
        return input; // Return the original string if the search value is not found.
    }
    return input.substring(0, index) + replace + input.substring(index + search.length);
};
exports.strReplaceLast = strReplaceLast;
var strSlug = function (input, separator) {
    if (separator === void 0) { separator = '-'; }
    return input
        .toLowerCase()
        .trim()
        .replace(/[^\w\s-]/g, '')
        .replace(/\s+/g, separator);
};
exports.strSlug = strSlug;
var strSnake = function (input, separator) {
    if (separator === void 0) { separator = '_'; }
    var snakeCase = input.replace(/([a-z\d])([A-Z])/g, '$1' + separator + '$2');
    return snakeCase.toLowerCase();
};
exports.strSnake = strSnake;
var strSquish = function (input) {
    return input.replace(/\s+/g, ' ').trim();
};
exports.strSquish = strSquish;
var strStart = function (input, prefix) {
    return input.startsWith(prefix) ? input : prefix + input;
};
exports.strStart = strStart;
var strStartsWith = function (input, search) {
    if (Array.isArray(search)) {
        return search.some(function (value) { return input.startsWith(value); });
    }
    return input.startsWith(search);
};
exports.strStartsWith = strStartsWith;
var strStudly = function (input) {
    return input
        .split(/[^a-zA-Z\d]+/)
        .map(function (word) { return word.charAt(0).toUpperCase() + word.slice(1).toLowerCase(); })
        .join('');
};
exports.strStudly = strStudly;
var strSwap = function (input, replacements) {
    var regex = new RegExp(Object.keys(replacements).join('|'), 'gi');
    return input.replace(regex, function (match) { return replacements[match.toLowerCase()] || match; });
};
exports.strSwap = strSwap;
var strTitle = function (input) {
    return input
        .toLowerCase()
        .replace(/(?:^|\s)\S/g, function (char) { return char.toUpperCase(); });
};
exports.strTitle = strTitle;
var strWordCount = function (input) {
    var words = input.trim().split(/\s+/).filter(function (word) { return word !== ''; });
    return words.length;
};
exports.strWordCount = strWordCount;
